;  ***************************************************************************
;  *                                                                         *
;  * File:   killmbr.asm          Version  1.11            Date: 2001-02-22  *
;  *                                                                         *
;  * Copyright (C) 1998-2001 by Kostis Netzwerkberatung                      *
;  * Talstr. 25, D-63322 Rdermark, Tel. +49 6074 881056, FAX 881058         *
;  * kosta@kostis.net (Kosta Kostis), http://www.kostis.net/                 *
;  *                                                                         *
;  ***************************************************************************
;  *                                                                         *
;  * Function:                                                               *
;  *     - kill/erase Master Boot Record resulting in a blank harddisk       *
;  *       so fdisk can be called using a script to create new partitions    *
;  *                                                                         *
;  ***************************************************************************
;  *                                                                         *
;  * Parameters:                                                             *
;  *     - /y skips confirmation (language specific)                         *
;  *     - /c, /d, /e, /f apply to drives 80h to 83h                         *
;  *                                                                         *
;  ***************************************************************************
;  *                                                                         *
;  * Comments:                                                               *
;  *                                                                         *
;  * With Microsoft's Macro Assembler 6.0 type:   ml /AT /cp killmbr.asm     *
;  * With Borland's   Turbo Assembler 2.5 type:   tasm /ml killmbr.asm       *
;  *                                              tlink /t killmbr.obj       *
;  *                                                                         *
;  ***************************************************************************
;  *                                                                         *
;  * History:                                                                *
;  *    2001-02-22: KK V1.11                                                 *
;  *        - added some I18N                                                *
;  *    1999-04-30: KK V1.10                                                 *
;  *        - changed default to US mbr (german mbr optional)                *
;  *    1998-12-09: KK V1.03                                                 *
;  *        - writes a valid German mbr instead of just zeroes               *
;  *    1998-11-29: KK V1.02                                                 *
;  *        - added switches /c, /d, /e, /f to select other drives           *
;  *    1998-11-08: KK V1.01                                                 *
;  *        - minor cosmetic changes                                         *
;  *    1998-11-06: KK V1.00                                                 *
;  *        - initial coding                                                 *
;  *                                                                         *
;  ***************************************************************************

.MODEL		TINY

.CONST

;  ***************************************************************************
;
;	BIOS disk service functions
;
;  ***************************************************************************

INT_DSK		EQU	13h		; Disk Interrupt
   DSK_WRITE	EQU	03h		;    write disk sector

DRIVE_C		EQU	80h		; Drive C: MBR is on h0/t0/s1
   MBR_HEAD	EQU	00h		; Head   0
   MBR_TRACK	EQU	00h		; Track  0
   MBR_SECTOR	EQU	01h		; Sector 1
   MBR_NUMSECT	EQU	01h		; only one sector
   SECTSIZE	EQU	200h		; sector size is 512 byte

;  ***************************************************************************
;
;	MS-DOS service functions
;
;  ***************************************************************************

INT_DOS		EQU	21h		; MS-DOS Service Interrupt
   DOS_GETKEY	EQU	01h		;    get key (with echo)
   DOS_PUTSTR	EQU	09h		;    display string terminated by '$'
   DOS_TERM	EQU	4Ch		;    terminate program

;  ***************************************************************************
;
;	offset constants in PSP (C like)
;
;  ***************************************************************************

ARGC		EQU	80h			; offset in PSP for arg count
ARGV		EQU	82h			; offset in PSP for arg value0

;  ***************************************************************************
;
;	personal C style macros to ease coding
;
;  ***************************************************************************

doscall		MACRO	Function		; call MS-DOS Function

		mov	ah,Function		;
		int	INT_DOS			;

		ENDM				;

puts		MACRO	String			; write string to stdout

		lea	dx,String		; must be terminated by a '$'
		doscall	DOS_PUTSTR		;

		ENDM				;

exit		MACRO	RetVal			; terminate program

		mov	al,RetVal		;
		jmp	terminate		; this label in this program

		ENDM				;

;  ***************************************************************************
;
;	Program Messages and Disk Write Buffer (mbr content)
;
;  ***************************************************************************

MSG_COPYRIGHT	DB	'Copyright (C) 1998-2001 by Kostis Netzwerkberatung',13,10
		DB	'Talstr. 25, D-63322 Rdermark, Tel. +49 6074 881056, FAX 881058',13,10
		DB	'kosta@kostis.net (Kosta Kostis), http://www.kostis.net/',13,10
		DB	'killmbr V1.11 (2001-02-22)',13,10
		DB	13,10

INCLUDE		msg-en.inc			; language specific stuff
; include msg-de.inc for German version

;  ***************************************************************************
;
;	Program Return Codes
;
;  ***************************************************************************

KILLMBR_DONE	EQU	0
KILLMBR_SYNTAX	EQU	1
KILLMBR_ABORT	EQU	2
KILLMBR_ERROR	EQU	3

;  ***************************************************************************
;
;	Main Program
;
;  ***************************************************************************

.CODE

		ORG	100h

start:		puts	MSG_COPYRIGHT		; display copyright information

		mov	bl,DRIVE_C		; drive C: is default

		mov	ah,cs:ARGC		; check command line parmeter
		cmp	ah,1			; one blank is allowed
		jle	ask			; no parameter given, so ask

		cmp	ah,3			; more than 2 characters?
		jg	syntax			; that can't be a valid option

		mov	ah,cs:ARGV		; check command line option
		cmp	ah,'/'			; valid switch character?
		je	getparam		;
		cmp	ah,'-'			; accept both '-' and '/'
		jne	syntax			;

getparam:	mov	ah,cs:ARGV+1		; check parameter
		and	ah,0DFh			; ignore case, simple parse

		cmp	ah,'Y'			; unattended killmbr
		je	killmbr			;

		cmp	ah,'C'			; drive parameter?
		jl	syntax			; can't be if < 'C'
		cmp	ah,'G'			; must be between C and F
		jg	syntax			; tell syntax then

		sub	ah,'C'			; get drive index
		or	bl,ah			; and add to default drive

ask:		puts	MSG_ASK			; shall we really do it?

		doscall	DOS_GETKEY		; get key

		push	ax			; save key in al
		puts	MSG_CR			; display cr/lf
		pop	ax			; restore key in al

		and	al,0DFh			; ignore case
		cmp	al,KEY_CONFIRM		; user confirms?
		jnz	abort			;

killmbr:	mov	ah,DSK_WRITE		; target is Master Boot Record
		mov	dh,MBR_HEAD		;
		mov	ch,MBR_TRACK		;
		mov	cl,MBR_SECTOR		;
		mov	al,MBR_NUMSECT		;
		mov	dl,bl			; set drive

		push	ds			;
		pop	es			;
		lea	bx,MBR_BUFFER		; ES:BX point to MBR_BUFFER

		int	INT_DSK			; write Master Boot Record

		cmp	ah,0			; sector successfully written?
		jnz	error			;

done:		puts	MSG_DONE		; success
		exit	KILLMBR_DONE		;

syntax:		puts	MSG_SYNTAX		; syntax error
		exit	KILLMBR_SYNTAX		;

abort:		puts	MSG_ABORT		; abort
		exit	KILLMBR_ABORT		;

error:		puts	MSG_ERROR		; sector write error
		exit	KILLMBR_ERROR		;

terminate:	doscall	DOS_TERM		; terminate program

		END	start			;
