/******************************************************************************
 *                                                                            *
 * File:   readsect.c         Version  1.00            Date: 1998-11-11       *
 *                                                                            *
 * Copyright (C) 1998 by Kostis Netzwerkberatung                              *
 * Talstr. 25, D-63322 Rdermark, Tel. +49 6074 881056, FAX 881058            *
 * kosta@kostis.net (Kosta Kostis), http://www.kostis.net/                    *
 *                                                                            *
 * You may use this software free of charge at your own risk.                 *
 *                                                                            *
 ******************************************************************************
 *                                                                            *
 * Function:                                                                  *
 *     - read sector, hexdump sector, display return code                     *
 *                                                                            *
 ******************************************************************************
 *                                                                            *
 * Parameters:                                                                *
 *     - drive, head, track, sector                                           *
 *                                                                            *
 ******************************************************************************
 *                                                                            *
 * History:                                                                   *
 *     1998-11-11: KK V1.00                                                   *
 *        - initial coding                                                    *
 *                                                                            *
 *****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <bios.h>

#define	SECTSIZE	512

typedef	unsigned char	uchar ;

void	txtput
(
	FILE	*Fout,
	uchar	ch
)
{
	if (ch < ' ')	// control codes should not be displayed directly
		fputc ('.', Fout) ;
	else
		fputc ((int) ch, Fout) ;
}

void	txtdump
(
	FILE	*Fout,
	uchar	*buffer,
	int	buflen
)
{
	fprintf (Fout, "  \"") ;
	while (buflen--)
		txtput (Fout, *buffer++) ;
	fprintf (Fout, "\"\n") ;
}

void	hexdump
(
	FILE	*Fout,
	uchar	*buffer,
	int	buflen,
	int	perline
)
{
	int	i ;
	int	line ;

	for (i = 0, line = 0 ; i < buflen ; ++i)
	{
		if (line == 0)
			fprintf (Fout, "%04X:", i) ;
		fprintf (Fout, " %02X", buffer [line]) ;
		++line ;
		if (line == perline)
		{
			txtdump (Fout, buffer, perline) ;
			line = 0 ;
			buffer += perline ;
		}
	}
}

void	main 
(
	int	argc,
	char	*argv	[]
)
{
	uchar	buffer	[SECTSIZE] ;
	int	drive ;
	int	head, track, sector ;
	int	rc ;

	if (argc != 5)
	{
		fprintf (stdout, "Syntax: %s drive head track sector\n",
			argv [0]) ;
		fprintf (stdout, "drive a:=0, drive b:=1, drive c:=128\n") ;
		return ;
	}

	memset (buffer, 0, SECTSIZE) ;

	drive	= atoi (argv [1]) ;
	head	= atoi (argv [2]) ;
	track	= atoi (argv [3]) ;
	sector	= atoi (argv [4]) ;

	fprintf (stdout, "Copyright (C) 1998 by Kostis Netzwerkberatung\n") ;
	fprintf (stdout, "Talstr. 25, D-63322 Rdermark, Tel. +49 6074 881056, FAX 881058\n") ;
	fprintf (stdout, "kosta@kostis.net (Kosta Kostis), http://www.kostis.net/\n") ;
	fprintf (stdout, "\nreadsect V1.00 (1998-11-11)\n\n") ;
	rc = biosdisk (_DISK_READ, drive, head, track, sector, 1, buffer) ;
	fprintf (stdout, "biosdisk (%d, %d, %d, %d, 1, buffer) = 0x%02X\n",
		drive, head, track, sector, rc) ;
	fputc ('\n', stdout) ;
	hexdump (stdout, buffer, SECTSIZE, 16) ;
}
